/* GNOME DB Server Library
 * Copyright (C) 2000 Rodrigo Moya
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "gda-server-impl.h"

static void gda_server_impl_init       (Gda_ServerImpl *server_impl);
static void gda_server_impl_class_init (Gda_ServerImplClass *klass);

static void gda_server_impl_destroy    (Gda_ServerImpl *server_impl);

static GList* server_list = NULL;

/*
 * Private functions
 */
static void
gda_server_impl_init (Gda_ServerImpl *server_impl)
{
  g_return_if_fail(IS_GDA_SERVER_IMPL(server_impl));

  server_impl->name = NULL;
  //  memset((void *) &server_impl, 0, sizeof(Gda_ServerImplFunctions));
}

static void
gda_server_impl_class_init (Gda_ServerImplClass *klass)
{
  GtkObjectClass* object_class = (GtkObjectClass *) klass;

  object_class->destroy = gda_server_impl_destroy;
}

static void
gda_server_impl_destroy (Gda_ServerImpl *server_impl)
{
  g_return_if_fail(IS_GDA_SERVER_IMPL(server_impl));

  if (server_impl->name) g_free((gpointer) server_impl->name);
}

GtkType
gda_server_impl_get_type (void)
{
  static guint type = 0;

  if (!type)
    {
      GtkTypeInfo info =
      {
	"Gda_ServerImpl",
	sizeof (Gda_ServerImpl),
	sizeof (Gda_ServerImplClass),
	(GtkClassInitFunc) gda_server_impl_class_init,
	(GtkObjectInitFunc) gda_server_impl_init,
	(GtkArgSetFunc)NULL,
        (GtkArgSetFunc)NULL,
      };
      type = gtk_type_unique(gtk_object_get_type(), &info);
    }
  return type;
}

/**
 * gda_server_impl_new
 * @name: name of the server
 * @functions: callback functions
 *
 * Create a new GDA provider implementation object. This function initializes
 * all the needed CORBA stuff, registers the server in the system's object
 * directory, and initializes all internal data. After successful return,
 * you've got a ready-to-go GDA provider. To start it, use
 * #gda_server_impl_start
 */
Gda_ServerImpl *
gda_server_impl_new (const gchar *name, Gda_ServerImplFunctions *functions)
{
  Gda_ServerImpl*    server_impl;
  PortableServer_POA root_poa;
  CORBA_char*        objref;
  CORBA_Object       name_service;

  g_return_val_if_fail(name != NULL, NULL);

  /* look for an already running instance */
  server_impl = gda_server_impl_find(name);
  if (server_impl) return server_impl;

  /* create provider instance */
  server_impl = GDA_SERVER_IMPL(gtk_type_new(gda_server_impl_get_type()));
  server_impl->name = g_strdup(name);
  if (functions)
    {
      memcpy((void *) &server_impl->functions,
	     (const void *) functions,
	     sizeof(Gda_ServerImplFunctions));
    }
  else g_warning(_("Starting provider %s with no implementation functions"), name);

  server_impl->connections = NULL;
  server_impl->is_running = FALSE;

  /* create CORBA connection factory */
  server_impl->ev = g_new0(CORBA_Environment, 1);
  server_impl->root_poa = (PortableServer_POA)
#if USING_OAF
    CORBA_ORB_resolve_initial_references(oaf_orb_get(), "RootPOA", server_impl->ev);
#else
    CORBA_ORB_resolve_initial_references(gnome_CORBA_ORB(), "RootPOA", server_impl->ev);
#endif
  gda_server_impl_exception(server_impl->ev);

  server_impl->connection_factory_obj = impl_GDA_ConnectionFactory__create(server_impl->root_poa,
									   server_impl->ev);
  gda_server_impl_exception(server_impl->ev);
#if USING_OAF
  objref = CORBA_ORB_object_to_string(oaf_orb_get()
#else
  objref = CORBA_ORB_object_to_string(gnome_CORBA_ORB(),
#endif
				      server_impl->connection_factory_obj,
				      server_impl->ev);
  gda_server_impl_exception(server_impl->ev);

#if USING_OAF
  oaf_active_server_register("IID", server_impl->connection_factory_obj);
#else
  name_service = gnome_name_service_get();
  if (CORBA_Object_is_nil(name_service, server_impl->ev))
    gda_log_error(_("Could not get CORBA name service object"));
  else
    {
      goad_server_register(name_service, server_impl->connection_factory_obj,
			   server_impl->name, "object", server_impl->ev);
    }
#endif
  gda_log_message(_("Registered with ID = %s"), objref);
  
  server_list = g_list_append(server_list, (gpointer) server_impl);
  return server_impl;
}

/**
 * gda_server_impl_find
 * @id: object id
 *
 * Searches the list of loaded server implementations by object activation
 * identification
 */
Gda_ServerImpl *
gda_server_impl_find (const gchar *id)
{
  GList* node;

  g_return_val_if_fail(id != NULL, NULL);

  node = g_list_first(server_list);
  while (node)
    {
      Gda_ServerImpl* server_impl = GDA_SERVER_IMPL(node->data);
      if (server_impl && !strcmp(server_impl->name, id))
	return server_impl;
      node = g_list_next(node);
    }
  return NULL;
}

/**
 * gda_server_impl_start
 * @server_impl: server implementation
 *
 * Starts the given GDA provider
 */
void
gda_server_impl_start (Gda_ServerImpl *server_impl)
{
  PortableServer_POAManager pm;

  g_return_if_fail(server_impl != NULL);
  g_return_if_fail(server_impl->is_running == FALSE);

  pm = PortableServer_POA__get_the_POAManager(server_impl->root_poa, server_impl->ev);
  gda_server_impl_exception(server_impl->ev);
  PortableServer_POAManager_activate(pm, server_impl->ev);
  gda_server_impl_exception(server_impl->ev);

  server_impl->is_running = TRUE;
#if USING_OAF
  CORBA_ORB_run(oaf_orb_get(), server_impl->ev);
#else
  CORBA_ORB_run(gnome_CORBA_ORB(), server_impl->ev);
#endif
}

/**
 * gda_server_impl_stop
 * @server_impl: server implementation
 *
 * Stops the given server implementation
 */
void
gda_server_impl_stop (Gda_ServerImpl *server_impl)
{
  g_return_if_fail(IS_GDA_SERVER_IMPL(server_impl));
  g_return_if_fail(server_impl->is_running);

#if USING_OAF
  CORBA_ORB_shutdown(oaf_orb_get(), TRUE, server_impl->ev);
  oaf_active_server_unregister("IID", server_impl->connection_factory_obj);
#else
  CORBA_ORB_shutdown(gnome_CORBA_ORB(), TRUE, server_impl->ev);
#endif
  server_impl->is_running = FALSE;
}

/*
 * Convenience functions
 */
gint
gda_server_impl_exception (CORBA_Environment *ev)
{
  g_return_val_if_fail(ev != NULL, -1);

  switch (ev->_major)
    {
    case CORBA_SYSTEM_EXCEPTION :
      gda_log_error(_("CORBA system exception %s"), CORBA_exception_id(ev));
      return -1;
    case CORBA_USER_EXCEPTION :
      gda_log_error(_("CORBA user exception: %s"), CORBA_exception_id( ev ));
      return -1;
    default:
      break;
    }
  return 0;
}

GDA_Error *
gda_server_impl_make_error_buffer (Gda_ServerConnection *cnc)
{
  gint       idx;
  GList*     ptr;
  GDA_Error* rc;

  g_return_val_if_fail(cnc != NULL, CORBA_OBJECT_NIL);

  rc  = CORBA_sequence_GDA_Error_allocbuf(g_list_length(cnc->errors));
  idx = 0;
  ptr = cnc->errors;

  while (ptr)
    {
      Gda_ServerError* e = (Gda_ServerError *) ptr->data;

      rc[idx].description = CORBA_string_dup(e->description);
      rc[idx].number = e->number;
      rc[idx].source = CORBA_string_dup(e->source);
#if 0
      rc->_buffer[idx].helpfile = CORBA_string_dup(e->helpfile);
      rc->_buffer[idx].helpctxt = CORBA_string_dup(e->helpctxt);
#endif
      rc[idx].sqlstate = CORBA_string_dup(e->sqlstate);
      rc[idx].nativeMsg   = CORBA_string_dup(e->native);
      gda_server_error_free(e);
      ptr = g_list_next(ptr);
      idx++;
    }
  g_list_free(cnc->errors);
  cnc->errors = NULL;
  return rc;
}


